#!/bin/bash
# Verify that a disk image created by PIPEDDR has not been altered.
# The disk image file contains a computed hash of the disk image data.
# This example script computes the hash the same way and compares the result
# with the hash in the file.
file=$1
# The "header" is the first 1024 bytes of the file.  It is structured with the
# EBCDIC characters first, a linefeed x0A, then the data in ASCII and an x0A.
# So, the header in ASCII is the second "line" in the header.
fileheader=$(head -c +1024 $file | cat | head -n 2 | tail -n 1)
if [ -z "$fileheader" ]; then
	echo "File $file is not in the correct format."
	exit
fi
# Show the header for verification.
echo "Header: $fileheader"
# Break file header into positional parameters
set $fileheader
# Get the length of the digest value.
len=${12}
# The digest is included in the last 1024 bytes of the file.  The first "len" bytes
# is the value in binary format.  The next "len*2" bytes is the value in 
# hexadecimal ascii format.  It is hard to work with binary here, so get the ascii.
hostdigest=$(tail -c 1024 $file | tail -c +$(($len+1)) | head -c $((len*2)))
# If the type is not CKSUM, it is one of the digest values.
if [ "$8" = "CKSUM" ]; then
	verifypgm="cksum"
# Convert the CRC-32 checksum and its length to decimal to match cksum output.
	hostdigest="$((16#${hostdigest:0:8})) $((16#${hostdigest:9:16}))"
else
# The type of digest is in the 11th word of the header.
# The verify program is "sha256sum" or "sha512sum" for example.  Add "sum" to the type.
	verifypgm=$(echo ${11} | tr [:upper:] [:lower:])sum
# Add filename "  -" to end to match command output
	hostdigest="$hostdigest  -"
fi
#
echo "Computing $verifypgm of the disk contents"
# Take the 1024 byte header and trailer off to compute digest or cksum
localdigest=$(tail -c +1025 $file | head -c -1024 | $verifypgm)
# Check that they match.
if [ "$hostdigest" != "$localdigest" ]; then
	echo $verifypgm "verification failed."
	echo "Value from PIPEDDR is:" $hostdigest
	echo "Local value is:" $localdigest
else
	echo "Digest values match"
fi
exit
